/*!
 * @file        main.c
 *
 * @brief       Main program body          
 *
 * @version     V1.0.0
 *
 * @date        2020-8-25
 */

#include "Board_APM32F072_MINI.h"
#include "stdio.h"
#include "stdint.h"
#include "apm32f0xx_gpio.h"
#include "apm32f0xx_misc.h"
#include "apm32f0xx_eint.h"
#include "apm32f0xx_rcm.h" 

/* Delay */
void Delay(uint32_t nTime);
/* Declared identifier TimingDelay*/
static uint32_t TimingDelay;

/*!
 * @brief       Main program
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
int main(void)
{
    APM_MINI_LEDInit(LED2);
    APM_MINI_LEDInit(LED3);
    
    APM_MINI_LEDOff(LED2);
    
    /** Configure the SysTick to generate a time base equal to 1 ms*/
    if(SysTick_Config(SystemCoreClock/1000))
    { 
        while (1);
    }
    
    while(1)
    {
        
        APM_MINI_LEDToggle(LED2);
        /** Delay 1 s*/
        Delay(1000);
        
        APM_MINI_LEDToggle(LED3);
        /** Delay 1 s*/
        Delay(1000);
        
    }
}

/*!
 * @brief       Delay
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
void Delay(uint32_t nTime)
{
    TimingDelay =nTime;
    
    while(TimingDelay != 0);
}

/*!
 * @brief       Enters Interrupt when TimingDelay's value changes
 *
 * @param       None
 *
 * @retval      None
 *
 * @note       
 */
void TimingDelay_Decrement(void)
{
    if (TimingDelay != 0x00)
    { 
        TimingDelay--;
    }
}

